import { CommandRegistry } from '@lumino/commands';
import { BoxPanel, Widget } from '@lumino/widgets';
import { PageConfig } from '@jupyterlab/coreutils';
import { ServerConnection, ServiceManager } from '@jupyterlab/services';
import { DocumentRegistry, Context } from '@jupyterlab/docregistry';
import { standardRendererFactories, RenderMimeRegistry } from '@jupyterlab/rendermime';
import { NotebookModelFactory, NotebookPanel, NotebookWidgetFactory, NotebookTracker, NotebookActions } from '@jupyterlab/notebook';
import { CodeMirrorEditorFactory, CodeMirrorMimeTypeService } from '@jupyterlab/codemirror';
import { Completer, CompleterModel, CompletionHandler, ConnectorProxy, KernelCompleterProvider } from '@jupyterlab/completer';
import { MathJaxTypesetter } from '@jupyterlab/mathjax2';
import getMarked from './render/marked';
import { requireLoader } from "@jupyter-widgets/html-manager";
import { WIDGET_MIMETYPE, WidgetRenderer } from "@jupyter-widgets/html-manager/lib/output_renderers";
import { NotebookCommands } from './NotebookCommands';
import ContentFactoryWithSidebar from './content/ContentFactoryWithSidebar';
import { IPyWidgetsClassicManager } from "../../jupyter/ipywidgets/IPyWidgetsClassicManager";
import { activateWidgetExtension } from "../../jupyter/ipywidgets/IPyWidgetsJupyterLabPlugin";
import { activatePlotlyWidgetExtension } from "../../jupyter/ipywidgets/plotly/JupyterlabPlugin";
export class NotebookAdapter {
    constructor(props, store, kernel) {
        this.props = props;
        this.kernel = kernel;
        this.boxPanel = new BoxPanel();
        this.boxPanel.addClass('dla-JupyterLab-Notebook');
        this.boxPanel.spacing = 0;
        this.loadNotebook = this.loadNotebook.bind(this);
        this.serverSettings = ServerConnection.makeSettings({
            appendToken: true,
            init: {
                credentials: "include",
                mode: 'cors',
            }
        });
        this.serviceManager = new ServiceManager({
            serverSettings: this.serverSettings,
        });
        this.store = store;
    }
    loadNotebook(notebookPath) {
        var _a;
        this.commandRegistry = new CommandRegistry();
        const useCapture = true;
        document.addEventListener('keydown', event => {
            this.commandRegistry.processKeydownEvent(event);
        }, useCapture);
        const rendermime = new RenderMimeRegistry({
            initialFactories: standardRendererFactories,
            latexTypesetter: new MathJaxTypesetter({
                url: PageConfig.getOption('mathjaxUrl'),
                config: PageConfig.getOption('mathjaxConfig'),
            }),
            markdownParser: getMarked(),
        });
        const documentRegistry = new DocumentRegistry({});
        const mimeTypeService = new CodeMirrorMimeTypeService();
        const editorServices = {
            factoryService: new CodeMirrorEditorFactory(),
            mimeTypeService,
        };
        const editorFactory = editorServices.factoryService.newInlineEditor;
        const contentFactory = this.props.sidebarComponent ?
            new ContentFactoryWithSidebar(this.props.sidebarComponent, this.commandRegistry, this.store, { editorFactory }) :
            new NotebookPanel.ContentFactory({ editorFactory });
        const tracker = new NotebookTracker({ namespace: 'notebook' });
        if (this.props.ipywidgets === 'classic') {
            this.iPyWidgetsClassicManager = new IPyWidgetsClassicManager({ loader: requireLoader });
            rendermime.addFactory({
                safe: false,
                mimeTypes: [WIDGET_MIMETYPE],
                createRenderer: (options) => new WidgetRenderer(options, this.iPyWidgetsClassicManager),
            }, 1);
        }
        else if (this.props.ipywidgets === 'lab') {
            const widgetRegistry = activateWidgetExtension(rendermime, tracker, null, null);
            activatePlotlyWidgetExtension(widgetRegistry);
        }
        const notebookWidgetFactory = new NotebookWidgetFactory({
            name: 'Notebook',
            modelName: 'notebook',
            fileTypes: ['notebook'],
            defaultFor: ['notebook'],
            preferKernel: true,
            canStartKernel: true,
            rendermime: rendermime,
            contentFactory: contentFactory,
            mimeTypeService: editorServices.mimeTypeService,
        });
        notebookWidgetFactory.widgetCreated.connect((sender, widget) => {
            widget.context.pathChanged.connect(() => {
                void tracker.save(widget);
            });
            void tracker.add(widget);
        });
        documentRegistry.addWidgetFactory(notebookWidgetFactory);
        const notebookModelFactory = new NotebookModelFactory({});
        documentRegistry.addModelFactory(notebookModelFactory);
        const context = new Context({
            manager: this.serviceManager,
            factory: notebookModelFactory,
            path: notebookPath,
            kernelPreference: {
                shouldStart: false,
                //        name: 'python3',
            }
        });
        this._notebookPanel = (_a = documentRegistry.getWidgetFactory('notebook')) === null || _a === void 0 ? void 0 : _a.createNew(context);
        const setupCompleter = (notebookPanel) => {
            const editor = notebookPanel.content.activeCell && notebookPanel.content.activeCell.editor;
            const sessionContext = notebookPanel.context.sessionContext;
            const completerModel = new CompleterModel();
            const completer = new Completer({ editor, model: completerModel });
            const completerTimeout = 1000;
            const provider = new KernelCompleterProvider();
            const connector = new ConnectorProxy({
                widget: notebookPanel,
                editor,
                session: sessionContext.session
            }, [provider], completerTimeout);
            const handler = new CompletionHandler({ completer, connector });
            void sessionContext.ready.then(() => {
                const provider = new KernelCompleterProvider();
                const connector = new ConnectorProxy({ widget: this._notebookPanel, editor, session: sessionContext.session }, [provider], completerTimeout);
                handler.connector = connector;
            });
            handler.editor = editor;
            notebookPanel.content.activeCellChanged.connect((sender, cell) => {
                handler.editor = cell && cell.editor;
            });
            completer.hide();
            Widget.attach(completer, document.body);
            return handler;
        };
        context.initialize(false).then(() => {
            var _a;
            (_a = this.kernel) === null || _a === void 0 ? void 0 : _a.getJupyterKernel().then((kernel) => {
                //    if (context.sessionContext.hasNoKernel) {
                context.sessionContext.changeKernel(kernel.model).then(() => {
                    const completerHandler = setupCompleter(this._notebookPanel);
                    NotebookCommands(this.commandRegistry, this._notebookPanel, completerHandler);
                });
                //      }
            });
        });
        if (this.props.ipywidgets === 'classic') {
            this._notebookPanel.sessionContext.kernelChanged.connect((sender, args) => {
                this.iPyWidgetsClassicManager.registerWithKernel(args.newValue);
            });
        }
        BoxPanel.setStretch(this._notebookPanel, 0);
        this.boxPanel.addWidget(this._notebookPanel);
        window.addEventListener('resize', () => {
            this._notebookPanel.update();
        });
        function getCurrent(args) {
            const widget = tracker.currentWidget;
            return widget;
        }
        function isEnabled() {
            return (tracker.currentWidget !== null);
        }
        this.commandRegistry.addCommand('run-selected-codecell', {
            label: 'Run Cell',
            execute: args => {
                const current = getCurrent(args);
                if (current) {
                    const { context, content } = current;
                    NotebookActions.run(content, context.sessionContext);
                }
            },
            isEnabled,
        });
    }
    get notebookPanel() {
        return this._notebookPanel;
    }
    get panel() {
        return this.boxPanel;
    }
    get manager() {
        return this.serviceManager;
    }
    get commands() {
        return this.commandRegistry;
    }
}
export default NotebookAdapter;
